// TermHire.cpp
// v1.05
// version for R4 release of the BeOS, for both Intel and PowerPC platforms
// release date 26 November 1998
// contact : brua@caids.u-strasbg.fr
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Directory.h>
#include <Window.h>
#include <StringView.h>
#include <File.h>
#include <Alert.h>
#include <Message.h>
#include <Errors.h>
#include <Path.h>

#include <TrackerAddOn.h> // this one is new in R3

#include "TermHire.h"
#include "TermHireUtils.h"
#include "TermHireApp.h"

// this is the function the Tracker call when the add-on is selected
// (see header in TrackerAddOn.h)
// dir_ref is the folder you have launched it from
// msg is a BMessage containing the references to BEntry's(Files/Folders)
// and "void *" is something unused yet (in R3 at least...)
void process_refs(entry_ref dir_ref, BMessage* msg, void*)
{
	long count;               // number of references stored in the BMessage
	long corrected_count;     // number of terminals that would be launched
	ulong type;               // type of the BMessage
	BPath path;               // folder where we launch the terminal
	bool fromfile=false;      // memorize if we launched a Terminal from a file
	char *argv[ARGC];


	if(!configure_argv(argv)) goto exit;

	msg->GetInfo("refs", &type, &count);
	corrected_count = 0;
	// first, we consider the case when the add-on has been selected
	// from the file menu of the Tracker window or by right-clicking
	// in the background of the window
	// i.e : in this case, there is no refs on the BMessage
	if (count == 0)
	{
		BEntry entry(&dir_ref);
		corrected_count = 1;
		entry.GetPath(&path);
		if(!LaunchTerminal(ARGC, (const char **)argv, (const char *)path.Path())) goto exit;
	}
	// some files have been selected, let's see...
	else
	{
		// open a popup if more than ASK_CONFIRMATION_LIMIT Terminal would be open
		if((corrected_count=number_asked(dir_ref,msg)) > ASK_CONFIRMATION_LIMIT)
		{
			char message[60];
			sprintf(message,"This action will open %d Terminal windows",corrected_count);
			BAlert *alert = new BAlert("TermHire Warning", 
				message, "Do it","Cancel",NULL,B_WIDTH_AS_USUAL,B_WARNING_ALERT);
			switch(alert->Go())
			{
				case 0 :
					break;
				case 1 : goto exit;
					break;
				default : break;
			}			
		}
		
		BDirectory dir;
		BEntry entry;
		// if there are datas on the BMessage, but not from the
		// directory type, exit...
		if (dir.SetTo(&dir_ref) != B_NO_ERROR) 
		{
			error("You must launch TermHire from folders.");
			return;
		}
		if(dir.GetEntry(&entry) != B_NO_ERROR)
		{
			error("Sorry, I cannot access the folder you specified");
			return;
		}
		// Here we browse each reference found in the BMessage
		entry_ref ref;
		BFile file;
		for (int ix=0; ix<count; ix++)
		{
			if(msg->FindRef("refs", ix, &ref) == B_NO_ERROR)
			// if the ref is a folder, launch a Terminal on this folder
			if(dir.SetTo(&ref) == B_NO_ERROR)
			{
				entry.SetTo(&ref);
				entry.GetPath(&path);
				if(!LaunchTerminal(ARGC, (const char **)argv, (const char *)path.Path())) goto exit;
			}
			// if the ref is not a folder
			else if(entry.SetTo(&ref) == B_NO_ERROR) 
			{
#ifdef MISTAKE_PROTECTION
				if(!fromfile)
				{
					fromfile=true;
					// launch a Terminal in the Parent folder
					// I ask for the parent directory of the entry
					entry.GetParent(&entry);
					entry.GetPath(&path);
					if(!LaunchTerminal(3, (const char **)argv, (const char *)path.Path())) goto exit;
				}
#else
				// launch a Terminal in the Parent folder
				// I ask for the parent directory of the entry
				entry.GetParent(&entry);
				entry.GetPath(&path);
				if(!LaunchTerminal(path.Path())) goto exit;
#endif
			}
		} // end of for(ix...
	} // end of file selection handling
exit:
	;
}