// TermHireUtils.cpp
// v1.05
// version for R4 release of the BeOS, for both Intel and PowerPC platforms
// release date 26 November 1998
// contact : brua@caids.u-strasbg.fr
#include <Alert.h>
#include <SupportDefs.h>
#include "TermHireUtils.h"
#include <Path.h>
#include <Directory.h>
#include <File.h>
#include <Mime.h>
#include <AppFileInfo.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

// 1st arg : the app which is launched (fallback path in case the MIME search fails)
// 2nd arg : tells it which shell to use. We use the SHELL variable,
//           but if it is not set, the default "/boot/beos/bin/sh" value is used.
// 3rd arg : tells the shell to be a login shell.

void error(const char *message)
{
	BAlert *alert = new BAlert("TermHire Error", message,
		"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
	alert->Go();
}

// this function creates the new environment
// which is a copy of the "orig" environment, 
// plus the environment variable defined in "new_var"
// maybe a "putenv" system call should do the trick (not avail in R3)
char **my_putenv(char **orig, char *new_var)
{
	int nb_lines=0,i;
	char **dest;
	
	while(orig[nb_lines]!=NULL) nb_lines++;
	dest=(char **)malloc((nb_lines+2)*sizeof(char **));
	
	// we just need to copy the pointers, since there will be another copy
	// when the app will be launched (I'm not really sure of that though)
	for(i=0;i<nb_lines;i++)
		dest[i]=orig[i];
		
	// now the new one is allocated	
	dest[nb_lines]=(char *)malloc(strlen(new_var)+1);
	strcpy(dest[nb_lines],new_var);
	dest[nb_lines+1]=NULL;
	
	return dest;
}

// this function launch a Terminal in the specified path
int LaunchTerminal(const int argc, const char **argv, const char *path)
{
	char **environ2,path2[MAXPATHLEN + 13];
	char errorstring[MAXPATHLEN + 40];

	// now, it formats the new environment context,
	// containing the variable FOLDER_PATH
	path2[0]='\0';
	strcat(path2,"FOLDER_PATH=");
	strcat(path2,path);
	
	// the "environ" variable is a string array that ends with a NULL
	// pointer and contains all the variables defined at boot time
	// in the environment
	environ2=my_putenv(environ,path2);

	thread_id tid;
	// we launch the new Terminal in a separate thread
	tid = load_image(argc, (const char **)argv, (const char **)environ2);
	if (tid > 0)
	{
		// the launch was successful, now run the thread
		resume_thread(tid);
		return 1;
	}
	else 
	{
		// the launch was unsuccessful
		sprintf(errorstring, "Error %d running %s", tid, argv[0]);
		error(errorstring);
		return B_ERROR;
	}
}

// gives back the number of terminals that would be launched if the
// BMessage msg is accepted
long number_asked(entry_ref dir_ref, BMessage *msg)
{
	BDirectory dir;
	BEntry entry;
	long launched = 0, // number of terminals really launched
	     count;        // number of refs in the BMessage
	ulong type;        // type of the BMessage
	bool fromfile = false;
	
	msg->GetInfo("refs", &type, &count);
	// if there are datas on the BMessage, but not from the
	// directory type, exit...
	if (dir.SetTo(&dir_ref) != B_NO_ERROR) return 0;
	if (dir.GetEntry(&entry) != B_NO_ERROR) return 0;
	// Here we browse each reference found in the BMessage
	entry_ref ref;
	BFile file;
	for (int ix=0; ix<count; ix++)
	{
		if(msg->FindRef("refs", ix, &ref) == B_NO_ERROR)
		// if the ref is a folder
		if(dir.SetTo(&ref) == B_NO_ERROR) launched++;
		// if the ref is not a folder
		else if(entry.SetTo(&ref) == B_NO_ERROR) 
		{
#ifndef MISTAKE_PROTECTION
			launched++;
#else
			if(!fromfile)
			{
				fromfile=true;
				launched++;
			}
#endif
		}
	}
	return launched;	
}

#ifdef TERMINAL_SNIFFER
// fill the application path corresponding to the signature
// return-codes :
// B_NO_ERROR        : nothing to tell about it, all ran well
// B_BAD_VALUE       : if the roster-acknowledged path isn't the correct app
// B_ENTRY_NOT_FOUND : no app with such a MIME type
// path is an already allocated string of B_PATH_NAME_LENGTH length
int get_app_from_sig(const char *app_signature, char *path)
{
	char test_sig[B_MIME_TYPE_LENGTH]; // result of afi.GetSignature
	BMimeType app_mime;                // BeOS signature-related info
	BFile app_file;
	BEntry app_entry;
	BPath app_path;
	entry_ref app_ref;
	BAppFileInfo app_fileinfo;

	// we ask for the Terminal executable through its mime type
	if(app_mime.SetTo(app_signature)!=B_NO_ERROR)
		return B_BAD_VALUE;
	if(app_mime.GetAppHint(&app_ref)!=B_NO_ERROR)
		return B_ENTRY_NOT_FOUND;


	// we check if the selected executable has the correct MIME type
	if(app_file.SetTo(&app_ref,B_READ_ONLY)!=B_NO_ERROR)
		return B_BAD_VALUE;
	if(app_fileinfo.SetTo(&app_file)!=B_NO_ERROR)
		return B_BAD_VALUE;
	if(app_fileinfo.GetSignature(test_sig)!=B_NO_ERROR)
		return B_BAD_VALUE;
	if(strcmp(test_sig,app_signature)!=0)
		return B_BAD_VALUE;

	// hmmm, I think this is really the Terminal exe I'm waiting for...
	if(app_entry.SetTo(&app_ref)!=B_NO_ERROR)
		return B_BAD_VALUE;
	if(app_entry.GetPath(&app_path)!=B_NO_ERROR)
		return B_BAD_VALUE;
	strcpy(path,app_path.Path());
	return B_NO_ERROR;
}
#endif

// check wether the file corresponding to "path" exists
bool valid_file(const char *path)
{
	BFile file;
	if(file.SetTo(path,B_READ_ONLY)==B_NO_ERROR)
		return true;
	else
		return false;
}

// search for Terminal and sh executables and put it in argv
// to prepare a Terminal launch. argv is an already-allocated
// char *[ARGC]
bool configure_argv(char **argv)
{
	char *termapp,*shellpath;

	termapp=(char*)malloc(sizeof(char)*MAXPATHLEN);
	argv[0]="/boot/beos/apps/Terminal";
#ifdef TERMINAL_SNIFFER
	if(get_app_from_sig(TERMINAL_SIGNATURE,termapp)==B_NO_ERROR)
		argv[0]=termapp;
#endif
	
	if(!valid_file(argv[0]))
	{
		error("Terminal app cannot be found");
		return false;
	}

	// we set up the shell path/name by grabbing it in the environment
	shellpath=getenv("SHELL");
	if(valid_file(shellpath))
		argv[1]=shellpath;
	else
	{
		argv[1]="/boot/beos/bin/sh";
		if(!valid_file(argv[1]))
		{
			error("Unable to find a valid shell, please define the SHELL variable");
			return false;
		}
	}
	argv[2]="-login";
	return true;
}
